#! /usr/bin/python3

import sys
import os
import subprocess

###############################################################################
def printUsageAndExit() :
  print("USAGE : %s (train | eval) (bash | oar) batchesDescription.py"%sys.argv[0],
      file=sys.stderr)
  exit(1)
###############################################################################

###############################################################################
def prepareExperiment(lang, template, expName) :
  subprocess.Popen("./prepareExperiment.sh %s %s %s"%(lang,template,expName),
      shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE).wait()
###############################################################################

###############################################################################
def launchTrain(mode, expName, arguments, launcher) :
  if launcher == "bash" :
    launchTrainBash(mode, expName, arguments)
  elif launcher == "oar" :
    launchTrainOar(mode, expName, arguments)
  else :
    printUsageAndExit()
###############################################################################

###############################################################################
def launchTrainBash(mode, expName, arguments) :
  devScore = "--devScore"
  subprocess.Popen("./train.sh %s bin/%s %s --silent %s"%(mode,expName,arguments,devScore),
    shell=True, stdout=open("%s.stdout"%expName,'w'), stderr=open("%s.stderr"%expName,'w'))
###############################################################################

###############################################################################
def launchTrainOar(mode, expName, arguments) :
  devScore = "--devScore"

  command = "oarsub"
  command += " -t besteffort"
  command += " -t idempotent"
  command += " -n train:%s"%expName
  command += " -E %s.stderr"%expName
  command += " -O %s.stdout"%expName
  command += " -p \"gpu IS NOT NULL\""
  command += " -l walltime=15:00:00"
  command += " \"" + "./train.sh %s bin/%s %s --silent %s"%(mode,expName,arguments,devScore) + "\""

  os.system(command)
###############################################################################

###############################################################################
def launchEval(mode, expName, launcher) :
  if launcher == "bash" :
    launchEvalBash(mode, expName)
  elif launcher == "oar" :
    launchEvalOar(mode, expName)
  else :
    printUsageAndExit()
###############################################################################

###############################################################################
def launchEvalBash(mode, expName) :
  subprocess.Popen("./evaluate.sh %s bin/%s --silent"%(mode,expName),
    shell=True, stdout=open("%s.stdout"%expName,'a'), stderr=open("%s.stderr"%expName,'a'))
###############################################################################

###############################################################################
def launchEvalOar(mode, expName) :
  command = "oarsub"
  command += " -t besteffort"
  command += " -n eval:%s"%expName
  command += " -E %s.stderr"%expName
  command += " -O %s.stdout"%expName
  command += " -p \"gpu IS NOT NULL\""
  command += " -l walltime=2:00:00"
  command += " \"" + "./evaluate.sh %s bin/%s --silent"%(mode,expName) + "\""

  os.system(command)
###############################################################################

###############################################################################
if __name__ == "__main__" :
  if len(sys.argv) != 4 :
    printUsageAndExit()

  mode = sys.argv[1]
  launcher = sys.argv[2]
  batchesDescription = sys.argv[3]

  if mode not in ["train","eval"] or launcher not in  ["bash","oar"] :
    printUsageAndExit()

  desc = __import__(os.path.splitext(batchesDescription)[0])

  for lang in desc.langs :
    for xp in desc.templatesExperiments :
      for i in range(desc.nbReplicas) :
        xp['lang'] = lang
        xp['expName'] = xp['expName'].split('.')[0]+"."+lang+"."+str(i)
        if mode == "train" :
          prepareExperiment(xp['lang'],xp['template'],xp['expName'])
          launchTrain(xp['mode'],xp['expName'],xp['arguments'],launcher)
        else :
          launchEval(xp['mode'],xp['expName'],launcher)

###############################################################################

